/*****************************  SuchBaum.java  ********************************/

import AlgoTools.IO;

/** Implementation eines binaeren Suchbaums.
 *  Verwendet wird das Interface Compare,
 *  das zwei Objects miteinander vergleicht.
 */

public class SuchBaum extends Baum {

    protected Compare comp;         // Vergleichs-Objekt

    public SuchBaum(Compare comp) { // erzeugt leeren Suchbaum

        super();
        this.comp = comp;
    }


    private SuchBaum find(Object x) { // sucht x im SuchBaum: liefert den
                                      // SuchBaum mit x in der Wurzel, ggf. leer
        if (empty())                     return this;
        if(comp.compare(x,value()) == 0) return this;
        if(comp.compare(x,value()) < 0)  return ((SuchBaum)left()).find(x);
        else                             return ((SuchBaum)right()).find(x);
    }


    public Object lookup(Object x) { // sucht x im SuchBaum: liefert null, wenn
                                     // x nicht gefunden wurde, sonst Objekt x
        return find(x).inhalt;
    }


    public boolean insert(Object x) { // fuegt x in den SuchBaum ein: liefert
                                      // true, wenn erfolgreich, sonst false.
        SuchBaum s = find(x);         // SuchBaum mit x in der Wurzel oder leer

        if (s.empty()) { // wenn leer, d.h. x noch nicht im SuchBaum enthalten:
            s.inhalt = x;                  // setzte Inhalt auf x
            s.links  = new SuchBaum(comp); // neuer leerer SuchBaum links
            s.rechts = new SuchBaum(comp); // neuer leerer SuchBaum rechts
            return true;
        }
        else return false;
    }

    public boolean delete(Object x) { // loescht x im SuchBaum: liefert true,
                                      // wenn erfolgreich geloescht, sonst false
        SuchBaum s = find(x);         // SuchBaum mit x in der Wurzel oder leer
        SuchBaum ersatz;              // Ersatzknoten

        if (s.empty()) return false;  // wenn x nicht gefunden: false
        else {                        // wenn x gefunden
            if      (s.left().empty())  ersatz = (SuchBaum)s.right();
            else if (s.right().empty()) ersatz = (SuchBaum)s.left();
            else {                    // Knoten mit x hat zwei Soehne
                ersatz = ((SuchBaum)s.left()).findMax(); // Maximum im linken
                s.inhalt = ersatz.inhalt;                // ersetze Inhalt
                s = ersatz;                              // zu ersetzen
                ersatz = (SuchBaum)ersatz.left();        // Ersatz: linker
            }
            s.inhalt = ersatz.inhalt; // ersetze die Komponenten
            s.links  = ersatz.links;
            s.rechts = ersatz.rechts;
            return true;
        }
    }

    private SuchBaum findMax() { // findet im nichtleeren SuchBaum das Maximum:
                           // liefert den SuchBaum mit dem Maximum in der Wurzel
        SuchBaum hilf = this;

        while (!hilf.right().empty()) hilf = (SuchBaum)hilf.right();
        return hilf;     // der rechteste Nachfahr von this
    }
}
